@@ -17,7 +17,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::RIFF;
 
-$VERSION = '1.23';
+$VERSION = '1.25';
 
 sub ProcessASF($$;$);
 sub ProcessContentDescription($$$);
@@ -154,6 +154,9 @@ my %advancedContentEncryption = (
     3 => 'CopyrightURL',
 );
 
+# Note: Many of these tags are similar to those in Image::ExifTool::Microsoft::Xtra
+#       and Image::ExifTool::WTV::Metadata
+# (tags in this table may have a leading "WM/" removed)
 %Image::ExifTool::ASF::ExtendedDescr = (
     PROCESS_PROC => \&ProcessExtendedContentDescription,
     GROUPS => { 2 => 'Video' },
@@ -238,7 +241,7 @@ my %advancedContentEncryption = (
     DVDID => {},
     EncodedBy => {},
     EncodingSettings => {},
-    EncodingTime => { Groups => { 2 => 'Time' } },
+    EncodingTime => { Groups => { 2 => 'Time' }, PrintConv => '$self->ConvertDateTime($val)' },
     Genre => {},
     GenreID => {},
     InitialKey => {},
@@ -260,7 +263,11 @@ my %advancedContentEncryption = (
     MediaIsSubtitled => {},
     MediaIsTape => {},
     MediaNetworkAffiliation => {},
-    MediaOriginalBroadcastDateTime => { Groups => { 2 => 'Time' } },
+    MediaOriginalBroadcastDateTime => {
+        Groups => { 2 => 'Time' },
+        ValueConv => '$val=~tr/-T/: /; $val',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
     MediaOriginalChannel => {},
     MediaStationCallSign => {},
     MediaStationName => {},
@@ -270,7 +277,11 @@ my %advancedContentEncryption = (
     OriginalArtist => {},
     OriginalFilename => 'OriginalFileName',
     OriginalLyricist => {},
-    OriginalReleaseTime => { Groups => { 2 => 'Time' } },
+    OriginalReleaseTime => {
+        Groups => { 2 => 'Time' },
+        ValueConv => '$val=~tr/-T/: /; $val',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
     OriginalReleaseYear => { Groups => { 2 => 'Time' } },
     ParentalRating => {},
     ParentalRatingReason => {},
@@ -380,8 +391,10 @@ my %advancedContentEncryption = (
     },
     32 => { Name => 'DataPackets',  Format => 'int64u' },
     40 => {
-        Name => 'PlayDuration',
+        Name => 'Duration',
         Format => 'int64u',
+        Notes => 'called PlayDuration by the ASF spec',
+        Priority => 0,
         ValueConv => '$val / 1e7',
         PrintConv => 'ConvertDuration($val)',
     },
@@ -863,7 +876,7 @@ Windows Media Audio (WMA) and Windows Media Video (WMV) files.
 
 =head1 AUTHOR
 
-Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
