@@ -14,6 +14,7 @@
 #               2019/07/02 - PH Added ability to read IMU CSV files
 #               2019/11/10 - PH Also write pitch to CameraElevationAngle
 #               2020/12/01 - PH Added ability to read DJI CSV log files
+#               2022/06/21 - PH Added ability to read Google Takeout JSON files
 #
 # References:   1) http://www.topografix.com/GPX/1/1/
 #               2) http://www.gpsinformation.org/dale/nmea.htm#GSA
@@ -28,7 +29,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:Public);
 use Image::ExifTool::GPS;
 
-$VERSION = '1.64';
+$VERSION = '1.68';
 
 sub JITTER() { return 2 }       # maximum time jitter
 
@@ -133,7 +134,7 @@ sub LoadTrackLog($$;$)
     local ($_, $/, *EXIFTOOL_TRKFILE);
     my ($et, $val) = @_;
     my ($raf, $from, $time, $isDate, $noDate, $noDateChanged, $lastDate, $dateFlarm);
-    my ($nmeaStart, $fixSecs, @fixTimes, $lastFix, %nmea, @csvHeadings);
+    my ($nmeaStart, $fixSecs, @fixTimes, $lastFix, %nmea, @csvHeadings, $sortFixes);
     my ($canCut, $cutPDOP, $cutHDOP, $cutSats, $e0, $e1, @tmp, $trackFile, $trackTime);
 
     unless (eval { require Time::Local }) {
@@ -210,13 +211,14 @@ sub LoadTrackLog($$;$)
         $raf->ReadLine($_) or last;
         # determine file format
         if (not $format) {
+            s/^\xef\xbb\xbf//;          # remove leading BOM if it exists
             if (/^<(\?xml|gpx)[\s>]/) { # look for XML or GPX header
                 $format = 'XML';
             # check for NMEA sentence
             # (must ONLY start with ones that have timestamps! eg. not GSA or PTNTHPR!)
             } elsif (/^.*\$([A-Z]{2}(RMC|GGA|GLL|ZDA)|PMGNTRK),/) {
                 $format = 'NMEA';
-                $nmeaStart = $2 || $1;    # save type of first sentence
+                $nmeaStart = $2 || $1;  # save type of first sentence
             } elsif (/^A(FLA|XSY|FIL)/) {
                 # (don't set format yet because we want to read HFDTE first)
                 $nmeaStart = 'B' ;
@@ -262,8 +264,10 @@ sub LoadTrackLog($$;$)
                         $param = 'time';
                     } elsif (/^(Pos)?Lat/i) {
                         $param = 'lat';
+                        /ref$/i and $param .= 'ref';
                     } elsif (/^(Pos)?Lon/i) {
                         $param = 'lon';
+                        /ref$/i and $param .= 'ref';
                     } elsif (/^(Pos)?Alt/i) {
                         $param = 'alt';
                     } elsif (/^(Angle)?(Heading|Track)/i) {
@@ -282,6 +286,10 @@ sub LoadTrackLog($$;$)
                     }
                 }
                 next;
+            } elsif (/"(timelineObjects|placeVisit|activitySegment|latitudeE7)":/) {
+                # Google Takeout JSON format
+                $format = 'JSON';
+                $sortFixes = 1; # (fixes are not all in order for this format)
             } else {
                 # search only first 50 lines of file for a valid fix
                 last if ++$skipped > 50;
@@ -453,7 +461,7 @@ DoneFix:    $isDate = 1;
 # (ExifTool enhancements allow for standard tag names or descriptions as the column headings,
 #  add support for time zones and flexible coordinates, and allow new DateTime and Shift columns)
 #
-            my ($param, $date, $secs);
+            my ($param, $date, $secs, %neg);
             foreach $param (@csvHeadings) {
                 my $val = shift @vals;
                 last unless defined $val;
@@ -479,6 +487,10 @@ DoneFix:    $isDate = 1;
                     }
                 } elsif ($param eq 'lat' or $param eq 'lon') {
                     $$fix{$param} = Image::ExifTool::GPS::ToDegrees($val, 1);
+                } elsif ($param eq 'latref') {
+                    $neg{lat} = 1 if $val =~ /^S/i;
+                } elsif ($param eq 'lonref') {
+                    $neg{lon} = 1 if $val =~ /^W/i;
                 } elsif ($param eq 'runtime') {
                     $date = $trackTime;
                     $secs = $val;
@@ -486,6 +498,11 @@ DoneFix:    $isDate = 1;
                     $$fix{$param} = $val;
                 }
             }
+            # make coordinate negative according to reference direction if necessary
+            foreach $param (keys %neg) {
+                next unless defined $$fix{$param};
+                $$fix{$param} = -abs($$fix{$param});
+            }
             if ($date and defined $secs and defined $$fix{lat} and defined $$fix{lon}) {
                 $time = $date + $secs;
                 $$has{alt} = 1 if defined $$fix{alt};
@@ -494,6 +511,19 @@ DoneFix:    $isDate = 1;
                 goto DoneFix;
             }
             next;
+        } elsif ($format eq 'JSON') {
+            # Google Takeout JSON format
+            if (/"(latitudeE7|longitudeE7|latE7|lngE7|timestamp)":\s*"?(.*?)"?,?\s*[\x0d\x0a]/) {
+                if ($1 eq 'timestamp') {
+                    $time = GetTime($2);
+                    goto DoneFix if $time and $$fix{lat} and $$fix{lon};
+                } elsif ($1 eq 'latitudeE7' or $1 eq 'latE7') {
+                    $$fix{lat} = $2 * 1e-7;
+                } else {
+                    $$fix{lon} = $2 * 1e-7;
+                }
+            }
+            next;
         }
         my (%fix, $secs, $date, $nmea);
         if ($format eq 'NMEA') {
@@ -528,7 +558,7 @@ DoneFix:    $isDate = 1;
         } elsif ($nmea eq 'RMC') {
             #  $GPRMC,092204.999,A,4250.5589,S,14718.5084,E,0.00,89.68,211200,,*25
             #  $GPRMC,093657.007,,3652.835020,N,01053.104094,E,1.642,,290913,,,A*0F
-            #  $GPRMC,hhmmss.sss,A/V,ddmm.mmmm,N/S,ddmmm.mmmm,E/W,spd(knots),dir(deg),DDMMYY,,*cs
+            #  $GPRMC,hhmmss.sss,A/V,ddmm.mmmm,N/S,dddmm.mmmm,E/W,spd(knots),dir(deg),DDMMYY,,*cs
             /^\$[A-Z]{2}RMC,(\d{2})(\d{2})(\d+(\.\d*)?),A?,(\d*?)(\d{1,2}\.\d+),([NS]),(\d*?)(\d{1,2}\.\d+),([EW]),(\d*\.?\d*),(\d*\.?\d*),(\d{2})(\d{2})(\d+)/ or next;
             next if $13 > 31 or $14 > 12 or $15 > 99;   # validate day/month/year
             $fix{lat} = (($5 || 0) + $6/60) * ($7 eq 'N' ? 1 : -1);
@@ -739,6 +769,8 @@ DoneFix:    $isDate = 1;
         $numPoints -= $cutHDOP;
         $numPoints -= $cutSats;
     }
+    # sort fixes if necessary
+    @fixTimes = sort { $a <=> $b } @fixTimes if $sortFixes;
     # mark first fix of the track
     while (@fixTimes) {
         $fix = $$points{$fixTimes[0]} or shift(@fixTimes), next;
@@ -1397,8 +1429,8 @@ This module is used by Image::ExifTool
 This module loads GPS track logs, interpolates to determine position based
 on time, and sets new GPS values for geotagging images.  Currently supported
 formats are GPX, NMEA RMC/GGA/GLL, KML, IGC, Garmin XML and TCX, Magellan
-PMGNTRK, Honeywell PTNTHPR, Winplus Beacon text, IMU CSV, DJI CSV, and
-Bramor gEO log files.
+PMGNTRK, Honeywell PTNTHPR, Bramor gEO, Winplus Beacon text, Google Takeout
+JSON, GPS/IMU CSV, DJI CSV, ExifTool CSV log files.
 
 Methods in this module should not be called directly.  Instead, the Geotag
 feature is accessed by writing the values of the ExifTool Geotag, Geosync
@@ -1412,7 +1444,7 @@ user-defined tag GPSRoll, must be active.
 
 =head1 AUTHOR
 
-Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
