@@ -12,7 +12,7 @@ use vars qw(%specialStruct %dateTimeInfo %stdXlatNS);
 
 use Image::ExifTool qw(:DataAccess :Utils);
 
-sub CheckXMP($$$);
+sub CheckXMP($$$;$);
 sub CaptureXMP($$$;$);
 sub SetPropertyPath($$;$$$$);
 
@@ -45,7 +45,7 @@ my $rdfClose = "</rdf:RDF>\n";
 my $xmpClose = "</x:xmpmeta>\n";
 my $pktCloseW =  "<?xpacket end='w'?>"; # writable by default
 my $pktCloseR =  "<?xpacket end='r'?>";
-my $noPad;
+my ($sp, $nl);
 
 #------------------------------------------------------------------------------
 # Get XMP opening tag (and set x:xmptk appropriately)
@@ -90,11 +90,11 @@ sub ValidateXMP($;$)
 
 #------------------------------------------------------------------------------
 # Validate XMP property
-# Inputs: 0) ExifTool ref, 1) validate hash ref
+# Inputs: 0) ExifTool ref, 1) validate hash ref, 2) attribute hash ref
 # - issues warnings if problems detected
-sub ValidateProperty($$)
+sub ValidateProperty($$;$)
 {
-    my ($et, $propList) = @_;
+    my ($et, $propList, $attr) = @_;
 
     if ($$et{XmpValidate} and @$propList > 2) {
         if ($$propList[0] =~ /^x:x[ma]pmeta$/ and
@@ -105,10 +105,23 @@ sub ValidateProperty($$)
                 if ($$propList[-1] =~ /^rdf:(Bag|Seq|Alt)$/) {
                     $et->Warn("Ignored empty $$propList[-1] list for $$propList[-2]", 1);
                 } else {
+                    if ($$propList[-2] eq 'rdf:Alt' and $attr) {
+                        my $lang = $$attr{'xml:lang'};
+                        if ($lang and @$propList >= 5) {
+                            my $langPath = join('/', @$propList[3..($#$propList-2)]);
+                            my $valLang = $$et{XmpValidateLangAlt} || ($$et{XmpValidateLangAlt} = { });
+                            $$valLang{$langPath} or $$valLang{$langPath} = { };
+                            if ($$valLang{$langPath}{$lang}) {
+                                $et->WarnOnce("Duplicate language ($lang) in lang-alt list: $langPath");
+                            } else {
+                                $$valLang{$langPath}{$lang} = 1;
+                            }
+                        }
+                    }
                     my $xmpValidate = $$et{XmpValidate};
                     my $path = join('/', @$propList[3..$#$propList]);
                     if (defined $$xmpValidate{$path}) {
-                        $et->Warn("Duplicate XMP property: $path") if defined $$xmpValidate{$path};
+                        $et->Warn("Duplicate XMP property: $path");
                     } else {
                         $$xmpValidate{$path} = 1;
                     }
@@ -124,7 +137,7 @@ sub ValidateProperty($$)
 
 #------------------------------------------------------------------------------
 # Check XMP date values for validity and format accordingly
-# Inputs: 1) date string
+# Inputs: 1) EXIF-format date string
 # Returns: XMP date/time string (or undef on error)
 sub FormatXMPDate($)
 {
@@ -152,12 +165,12 @@ sub FormatXMPDate($)
 
 #------------------------------------------------------------------------------
 # Check XMP values for validity and format accordingly
-# Inputs: 0) ExifTool object ref, 1) tagInfo hash ref, 2) raw value ref
+# Inputs: 0) ExifTool object ref, 1) tagInfo hash ref, 2) raw value ref, 3) conversion type
 # Returns: error string or undef (and may change value) on success
 # Note: copies structured information to avoid conflicts with calling code
-sub CheckXMP($$$)
+sub CheckXMP($$$;$)
 {
-    my ($et, $tagInfo, $valPtr) = @_;
+    my ($et, $tagInfo, $valPtr, $convType) = @_;
 
     if ($$tagInfo{Struct}) {
         require 'Image/ExifTool/XMPStruct.pl';
@@ -237,9 +250,12 @@ sub CheckXMP($$$)
         return "Invalid date/time (use YYYY:mm:dd HH:MM:SS[.ss][+/-HH:MM|Z])" unless $newDate;
         $$valPtr = $newDate;
     } elsif ($format eq 'boolean') {
+        # (allow lower-case 'true' and 'false' if not setting PrintConv value)
         if (not $$valPtr or $$valPtr =~ /false/i or $$valPtr =~ /^no$/i) {
-            $$valPtr = 'False';
-        } else {
+            if (not $$valPtr or $$valPtr ne 'false' or not $convType or $convType eq 'PrintConv') {
+                $$valPtr = 'False';
+            }
+        } elsif ($$valPtr ne 'true' or not $convType or $convType eq 'PrintConv') {
             $$valPtr = 'True';
         }
     } elsif ($format eq '1') {
@@ -282,6 +298,8 @@ sub SetPropertyPath($$;$$$$)
         $flatInfo = $$tagTablePtr{$flatID};
         if ($flatInfo) {
             return if $$flatInfo{PropertyPath};
+        } elsif (@$propList > 50) {
+            return; # avoid deep recursion
         } else {
             # flattened tag doesn't exist, so create it now
             # (could happen if we were just writing a structure)
@@ -324,9 +342,8 @@ sub SetPropertyPath($$;$$$$)
     # add required properties if this is a list
     push @propList, "rdf:$listType", 'rdf:li 10' if $listType and $listType ne '1';
     # set PropertyPath for all flattened tags of this structure if necessary
-    # (note: don't do this for variable-namespace structures (undef NAMESPACE))
     my $strTable = $$tagInfo{Struct};
-    if ($strTable and $$strTable{NAMESPACE} and not ($parentID and
+    if ($strTable and not ($parentID and
         # must test NoSubStruct flag to avoid infinite recursion
         (($$tagTablePtr{$parentID} and $$tagTablePtr{$parentID}{NoSubStruct}) or
         length $parentID > 500))) # avoid deep recursion
@@ -491,10 +508,10 @@ sub ConformPathToNamespace($$)
     my $prop;
     foreach $prop (@propList) {
         my ($ns, $tag) = $prop =~ /(.+?):(.*)/;
-        next if $$nsUsed{$ns};
+        next if not defined $ns or $$nsUsed{$ns};
         my $uri = $nsURI{$ns};
         unless ($uri) {
-            warn "No URI for namepace prefix $ns!\n";
+            warn "No URI for namespace prefix $ns!\n";
             next;
         }
         my $ns2;
@@ -543,6 +560,36 @@ sub AddStructType($$$$;$)
     }
 }
 
+#------------------------------------------------------------------------------
+# Hack to use XMP writer for SphericalVideoXML
+# Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
+# Returns: SphericalVideoXML data
+sub WriteGSpherical($$$)
+{
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    $$dirInfo{Compact} = 1,
+    my $dataPt = $$dirInfo{DataPt};
+    if ($dataPt and $$dataPt) {
+        # make it look like XMP for writing
+        my $buff = $$dataPt;
+        $buff =~ s/<rdf:SphericalVideo/<?xpacket begin='.*?' id='W5M0MpCehiHzreSzNTczkc9d'?>\n<x:xmpmeta xmlns:x='adobe:ns:meta\/'><rdf:RDF/;
+        $buff =~ s/\s*xmlns:GSpherical/>\n<rdf:Description xmlns:GSpherical/s;
+        $buff =~ s/<\/rdf:SphericalVideo>/<\/rdf:Description>/;
+        $buff .= "</rdf:RDF></x:xmpmeta><?xpacket end='w'?>";
+        $$dirInfo{DataPt} = \$buff;
+        $$dirInfo{DirLen} = length($buff) - ($$dirInfo{DirStart} || 0);
+    }
+    my $xmp = Image::ExifTool::XMP::WriteXMP($et, $dirInfo, $tagTablePtr);
+    if ($xmp) {
+        # change back to rdf:SphericalVideo structure
+        $xmp =~ s/^<\?xpacket begin.*?<rdf:RDF/<rdf:SphericalVideo\n/s;
+        $xmp =~ s/>\s*<rdf:Description rdf:about=''\s*/\n /;
+        $xmp =~ s/\s*<\/rdf:Description>\s*(<\/rdf:RDF>)/\n<\/rdf:SphericalVideo>$1/s;
+        $xmp =~ s/\s*<\/rdf:RDF>\s*<\/x:xmpmeta>.*//s;
+    }
+    return $xmp;
+}
+
 #------------------------------------------------------------------------------
 # Utility routine to encode data in base64
 # Inputs: 0) binary data string, 1) flag to avoid inserting newlines
@@ -604,13 +651,12 @@ sub LimitXMPSize($$$$$$)
     push @$startPt, length($$dataPt);  # add end offset to list
     my $newData = substr($$dataPt, 0, $$startPt[0]);
     my $guid = '0' x 32;
-    my $sp = $noPad ? '' : ' ';
     # write the required xmpNote:HasExtendedXMP property
-    $newData .= "\n$sp<$rdfDesc rdf:about='${about}'\n$sp${sp}xmlns:xmpNote='$nsURI{xmpNote}'";
-    if ($et->Options('XMPShorthand')) {
+    $newData .= "$nl$sp<$rdfDesc rdf:about='${about}'\n$sp${sp}xmlns:xmpNote='$nsURI{xmpNote}'";
+    if ($$et{OPTIONS}{Compact}{Shorthand}) {
         $newData .= "\n$sp${sp}xmpNote:HasExtendedXMP='${guid}'/>\n";
     } else {
-        $newData .= ">\n$sp$sp<xmpNote:HasExtendedXMP>$guid</xmpNote:HasExtendedXMP>\n$sp</$rdfDesc>\n";
+        $newData .= ">$nl$sp$sp<xmpNote:HasExtendedXMP>$guid</xmpNote:HasExtendedXMP>$nl$sp</$rdfDesc>\n";
     }
 
     my ($i, %descSize, $start);
@@ -645,49 +691,49 @@ sub LimitXMPSize($$$$$$)
 # Close out bottom-level property
 # Inputs: 0) current property path list ref, 1) longhand properties at each resource
 #         level, 2) shorthand properties at each resource level, 3) resource flag for
-#         each property path level (set only if XMPShorthand is enabled)
+#         each property path level (set only if Shorthand is enabled)
 sub CloseProperty($$$$)
 {
     my ($curPropList, $long, $short, $resFlag) = @_;
 
     my $prop = pop @$curPropList;
     $prop =~ s/ .*//;       # remove list index if it exists
-    my $pad = $noPad ? '' : ' ' x (scalar(@$curPropList) + 1);
+    my $pad = $sp x (scalar(@$curPropList) + 1);
     if ($$resFlag[@$curPropList]) {
         # close this XMP structure with possible shorthand properties
         if (length $$short[-1]) {
             if (length $$long[-1]) {
                 # require a new Description if both longhand and shorthand properties
-                $$long[-2] .= ">\n$pad<$rdfDesc";
-                $$short[-1] .= ">\n";
-                $$long[-1] .= "$pad</$rdfDesc>\n";
+                $$long[-2] .= ">$nl$pad<$rdfDesc";
+                $$short[-1] .= ">$nl";
+                $$long[-1] .= "$pad</$rdfDesc>$nl";
             } else {
                 # simply close empty property if all shorthand
-                $$short[-1] .= "/>\n";
+                $$short[-1] .= "/>$nl";
             }
         } else {
             # use "parseType" instead of opening a new Description
             $$long[-2] .= ' rdf:parseType="Resource"';
-            $$short[-1] = length $$long[-1] ? ">\n" : "/>\n";
+            $$short[-1] = length $$long[-1] ? ">$nl" : "/>$nl";
         }
-        $$long[-1] .= "$pad</$prop>\n" if length $$long[-1];
+        $$long[-1] .= "$pad</$prop>$nl" if length $$long[-1];
         $$long[-2] .= $$short[-1] . $$long[-1];
         pop @$short;
         pop @$long;
     } elsif (defined $$resFlag[@$curPropList]) {
         # close this top level Description with possible shorthand values
         if (length $$long[-1]) {
-            $$long[-2] .= $$short[-1] . ">\n" . $$long[-1] . "$pad</$prop>\n";
+            $$long[-2] .= $$short[-1] . ">$nl" . $$long[-1] . "$pad</$prop>$nl";
         } else {
-            $$long[-2] .= $$short[-1] . "/>\n"; # empty element (ie. all shorthand)
+            $$long[-2] .= $$short[-1] . "/>$nl"; # empty element (ie. all shorthand)
         }
         $$short[-1] = $$long[-1] = '';
     } else {
         # close this property (no chance of shorthand)
-        $$long[-1] .= "$pad</$prop>\n";
+        $$long[-1] .= "$pad</$prop>$nl";
         unless (@$curPropList) {
             # add properties now that this top-level Description is complete
-            $$long[-2] .= ">\n" . $$long[-1];
+            $$long[-2] .= ">$nl" . $$long[-1];
             $$long[-1] = '';
         }
     }
@@ -696,8 +742,8 @@ sub CloseProperty($$$$)
 
 #------------------------------------------------------------------------------
 # Write XMP information
-# Inputs: 0) ExifTool object reference, 1) source dirInfo reference,
-#         2) [optional] tag table reference
+# Inputs: 0) ExifTool ref, 1) source dirInfo ref (with optional WriteGroup),
+#         2) [optional] tag table ref
 # Returns: with tag table: new XMP data (may be empty if no XMP data) or undef on error
 #          without tag table: 1 on success, 0 if not valid XMP file, -1 on write error
 # Notes: May set dirInfo InPlace flag to rewrite with specified DirLen (=2 to allow larger)
@@ -715,11 +761,10 @@ sub WriteXMP($$;$)
     my $xmpFile = (not $tagTablePtr);   # this is an XMP data file if no $tagTablePtr
     # prefer XMP over other metadata formats in some types of files
     my $preferred = $xmpFile || ($$et{PreferredGroup} and $$et{PreferredGroup} eq 'XMP');
-    my $verbose = $et->Options('Verbose');
-    my $compact = $et->Options('Compact') || 0;
+    my $verbose = $$et{OPTIONS}{Verbose};
+    my %compact = ( %{$$et{OPTIONS}{Compact}} ); # (make a copy so we can change settings)
     my $dirLen = $$dirInfo{DirLen};
     $dirLen = length($$dataPt) if not defined $dirLen and $dataPt;
-    $noPad = ($compact > 1);
 #
 # extract existing XMP information into %capture hash
 #
@@ -736,6 +781,9 @@ sub WriteXMP($$;$)
     delete $$et{XMP_IS_XML};
     delete $$et{XMP_IS_SVG};
 
+    # set current padding characters
+    ($sp, $nl) = ($compact{NoIndent} ? '' : ' ', $compact{NoNewline} ? '' : "\n");
+
     # get value for new rdf:about
     my $tagInfo = $Image::ExifTool::XMP::rdf{about};
     if (defined $$et{NEW_VALUE}{$tagInfo}) {
@@ -816,7 +864,7 @@ sub WriteXMP($$;$)
 #
     if (%{$$et{DEL_GROUP}} and (grep /^XMP-.+$/, keys %{$$et{DEL_GROUP}} or
         # (logic is a bit more complex for group names in exiftool XML files)
-        grep m{^http://ns.exiftool.ca/}, values %nsUsed))
+        grep m{^http://ns.exiftool.(?:ca|org)/}, values %nsUsed))
     {
         my $del = $$et{DEL_GROUP};
         my $path;
@@ -827,7 +875,7 @@ sub WriteXMP($$;$)
             $ns = $stdXlatNS{$ns} if $stdXlatNS{$ns};
             my ($grp, @g);
             # no "XMP-" added to most groups in exiftool RDF/XML output file
-            if ($nsUsed{$ns} and (@g = ($nsUsed{$ns} =~ m{^http://ns.exiftool.ca/(.*?)/(.*?)/}))) {
+            if ($nsUsed{$ns} and (@g = ($nsUsed{$ns} =~ m{^http://ns.exiftool.(?:ca|org)/(.*?)/(.*?)/}))) {
                 if ($g[1] =~ /^\d/) {
                     $grp = "XML-$g[0]";
                     #(all XML-* groups stored as uppercase DEL_GROUP key)
@@ -861,10 +909,12 @@ sub WriteXMP($$;$)
     # get hash of all information we want to change
     # (sorted by tag name so alternate languages come last, but with structures
     # first so flattened tags may be used to override individual structure elements)
-    my @tagInfoList;
+    my (@tagInfoList, $delLangPath, %delLangPaths, %delAllLang, $firstNewPath);
+    my $writeGroup = $$dirInfo{WriteGroup};
     foreach $tagInfo (sort ByTagName $et->GetNewTagInfoList()) {
         next unless $et->GetGroup($tagInfo, 0) eq 'XMP';
         next if $$tagInfo{Name} eq 'XMP'; # (ignore full XMP block if we didn't write it already)
+        next if $writeGroup and $writeGroup ne $$et{NEW_VALUE}{$tagInfo}{WriteGroup};
         if ($$tagInfo{Struct}) {
             unshift @tagInfoList, $tagInfo;
         } else {
@@ -872,6 +922,7 @@ sub WriteXMP($$;$)
         }
     }
     foreach $tagInfo (@tagInfoList) {
+        my @delPaths;   # list of deleted paths
         my $tag = $$tagInfo{TagID};
         my $path = GetPropertyPath($tagInfo);
         unless ($path) {
@@ -987,9 +1038,33 @@ sub WriteXMP($$;$)
         my $nvHash = $et->GetNewValueHash($tagInfo);
         my $overwrite = $et->IsOverwriting($nvHash);
         my $writable = $$tagInfo{Writable} || '';
-        my (%attrs, $deleted, $added, $existed);
+        my (%attrs, $deleted, $added, $existed, $newLang);
+        # set up variables to save/restore paths of deleted lang-alt tags
+        if ($writable eq 'lang-alt') {
+            $newLang = lc($$tagInfo{LangCode} || 'x-default');
+            if ($delLangPath and $delLangPath eq $path) {
+                # restore paths of deleted entries for this language
+                @delPaths = @{$delLangPaths{$newLang}} if $delLangPaths{$newLang};
+            } else {
+                undef %delLangPaths;
+                $delLangPath = $path;   # base path for deleted lang-alt tags
+                undef %delAllLang;
+                undef $firstNewPath;    # reset first path for new lang-alt tag
+            }
+            if (%delAllLang) {
+                # add missing paths to delete list for entries where all languages were deleted
+                my ($prefix, $reSort);
+                foreach $prefix (keys %delAllLang) {
+                    next if grep /^$prefix/, @delPaths;
+                    push @delPaths, "${prefix}10";
+                    $reSort = 1;
+                }
+                @delPaths = sort @delPaths if $reSort;
+            }
+        }
         # delete existing entry if necessary
         if ($isStruct) {
+            # delete all structure (or pseudo-structure) elements
             require 'Image/ExifTool/XMPStruct.pl';
             ($deleted, $added, $existed) = DeleteStruct($et, \%capture, \$path, $nvHash, \$changed);
             next unless $deleted or $added or $et->IsOverwriting($nvHash);
@@ -999,7 +1074,7 @@ sub WriteXMP($$;$)
             # take attributes from old values if they exist
             %attrs = %{$$cap[1]};
             if ($overwrite) {
-                my ($delPath, $oldLang, $delLang, $addLang, @matchingPaths);
+                my ($oldLang, $delLang, $addLang, @matchingPaths, $langPathPat, %langsHere);
                 # check to see if this is an indexed list item
                 if ($path =~ / /) {
                     my $pp;
@@ -1008,17 +1083,24 @@ sub WriteXMP($$;$)
                 } else {
                     push @matchingPaths, $path;
                 }
+                my $oldOverwrite = $overwrite;
                 foreach $path (@matchingPaths) {
                     my ($val, $attrs) = @{$capture{$path}};
                     if ($writable eq 'lang-alt') {
+                        # get original language code (lc for comparisons)
+                        $oldLang = lc($$attrs{'xml:lang'} || 'x-default');
+                        # revert to original overwrite flag if this is in a different structure
+                        if (not $langPathPat or $path !~ /^$langPathPat$/) {
+                            $overwrite = $oldOverwrite;
+                            ($langPathPat = $path) =~ s/\d+$/\\d+/;
+                        }
+                        # remember languages in this lang-alt list
+                        $langsHere{$langPathPat}{$oldLang} = 1;
                         unless (defined $addLang) {
                             # add to lang-alt list by default if creating this tag from scratch
                             $addLang = $$nvHash{IsCreating} ? 1 : 0;
                         }
-                        # get original language code (lc for comparisons)
-                        $oldLang = lc($$attrs{'xml:lang'} || 'x-default');
                         if ($overwrite < 0) {
-                            my $newLang = lc($$tagInfo{LangCode} || 'x-default');
                             next unless $oldLang eq $newLang;
                             # only add new tag if we are overwriting this one
                             # (note: this won't match if original XML contains CDATA!)
@@ -1058,9 +1140,23 @@ sub WriteXMP($$;$)
                     }
                     # save attributes and path from first deleted property
                     # so we can replace it exactly
-                    unless ($delPath) {
-                        %attrs = %$attrs;
-                        $delPath = $path;
+                    %attrs = %$attrs unless @delPaths;
+                    if ($writable eq 'lang-alt') {
+                        $langsHere{$langPathPat}{$oldLang} = 0; # (lang was deleted)
+                    }
+                    # save deleted paths so we can replace the same elements
+                    # (separately for each language of a lang-alt list)
+                    if ($writable ne 'lang-alt' or $oldLang eq $newLang) {
+                        push @delPaths, $path;
+                    } else {
+                        $delLangPaths{$oldLang} or $delLangPaths{$oldLang} = [ ];
+                        push @{$delLangPaths{$oldLang}}, $path;
+                    }
+                    # keep track of paths where we deleted all languages of a lang-alt tag
+                    if ($delLang) {
+                        my $p;
+                        ($p = $path) =~ s/\d+$//;
+                        $delAllLang{$p} = 1;
                     }
                     # delete this tag
                     delete $capture{$path};
@@ -1072,14 +1168,28 @@ sub WriteXMP($$;$)
                         delete $capture{"$pp/rdf:type"} if @a == 1;
                     }
                 }
-                next unless $delPath or $$tagInfo{List} or $addLang;
-                if ($delPath) {
-                    $path = $delPath;
+                next unless @delPaths or $$tagInfo{List} or $addLang;
+                if (@delPaths) {
+                    $path = shift @delPaths;
+                    # make sure new path is unique
+                    while ($capture{$path}) {
+                        last unless $path =~ s/ \d(\d+)$/' '.length($1+1).($1+1)/e;
+                    }
                     $deleted = 1;
                 } else {
                     # don't change tag if we couldn't delete old copy
                     # unless this is a list or an lang-alt tag
                     next unless $$tagInfo{List} or $oldLang;
+                    # avoid adding duplicate entry to lang-alt in a list
+                    if ($writable eq 'lang-alt' and %langsHere) {
+                        foreach (sort keys %langsHere) {
+                            next unless $path =~ /^$_$/;
+                            last unless $langsHere{$_}{$newLang};
+                            $path =~ /(.* )\d(\d+)(.*? \d+)$/ or $et->Error('Internal error writing lang-alt list'), last;
+                            my $nxt = $2 + 1;
+                            $path = $1 . length($nxt) . ($nxt) . $3; # step to next index
+                        }
+                    }
                     # (match last index to put in same lang-alt list for Bag of lang-alt items)
                     $path =~ m/.* (\d+)/g or warn "Internal error: no list index!\n", next;
                     $added = $1;
@@ -1087,7 +1197,15 @@ sub WriteXMP($$;$)
             } else {
                 # we are never overwriting, so we must be adding to a list
                 # match the last index unless this is a list of lang-alt lists
-                my $pat = $writable eq 'lang-alt' ? '.* (\d+)(.*? \d+)' : '.* (\d+)';
+                my $pat = '.* (\d+)';
+                if ($writable eq 'lang-alt') {
+                    if ($firstNewPath) {
+                        $path = $firstNewPath;
+                        $overwrite = 1; # necessary to put x-default entry first below
+                    } else {
+                        $pat = '.* (\d+)(.*? \d+)';
+                    }
+                }
                 if ($path =~ m/$pat/g) {
                     $added = $1;
                     # set position to end of matching index number
@@ -1104,7 +1222,7 @@ sub WriteXMP($$;$)
                     $$tagInfo{LangCode} eq 'x-default'))
                 {
                     my $saveCap = $capture{$path};
-                    for (;;) {
+                    while ($saveCap) {
                         my $p = $path;
                         substr($p, $pos, $len) = length($nxt) . $nxt;
                         # increment index in the path of the existing item
@@ -1116,10 +1234,9 @@ sub WriteXMP($$;$)
                     }
                 } else {
                     # add to end of list
-                    for (;;) {
+                    while ($capture{$path}) {
                         my $try = length($nxt) . $nxt;
                         substr($path, $pos, $len) = $try;
-                        last unless $capture{$path};
                         $len = length $try;
                         ++$nxt;
                     }
@@ -1142,8 +1259,10 @@ sub WriteXMP($$;$)
         my @newValues = $et->GetNewValue($nvHash) or next;
 
         # set language attribute for lang-alt lists
-        $attrs{'xml:lang'} = $$tagInfo{LangCode} || 'x-default' if $writable eq 'lang-alt';
-
+        if ($writable eq 'lang-alt') {
+            $attrs{'xml:lang'} = $$tagInfo{LangCode} || 'x-default';
+            $firstNewPath = $path if defined $added;  # save path of first lang-alt tag added
+        }
         # add new value(s) to %capture hash
         my $subIdx;
         for (;;) {
@@ -1164,7 +1283,9 @@ sub WriteXMP($$;$)
                         $et->Warn("$grp:$$tagInfo{Name} written as a literal because value is not a valid URI", 1);
                         # fall through to write as a string literal
                     }
-                    delete $attrs{'rdf:resource'};  # (remove existing resource if necessary)
+                    # remove existing value and/or resource attribute if they exist
+                    delete $attrs{'rdf:value'};
+                    delete $attrs{'rdf:resource'};
                     $capture{$path} = [ $newValue, \%attrs ];
                     last;
                 }
@@ -1182,15 +1303,23 @@ sub WriteXMP($$;$)
             # match last index except for lang-alt items where we want to put each
             # item in a different lang-alt list (so match the 2nd-last for these)
             my $pat = $writable eq 'lang-alt' ? '.* (\d+)(.*? \d+)' : '.* (\d+)';
-            $path =~ m/$pat/g or warn("Internal error: no list index for $tag!\n"), next;
+            pos($path) = 0;
+            $path =~ m/$pat/g or warn("Internal error: no list index for $tag ($path) ($pat)!\n"), next;
             my $idx = $1;
             my $len = length $1;
             my $pos = pos($path) - $len - ($2 ? length $2 : 0);
-            # generate unique list sub-indices to store additional values in sequence
+            # use sub-indices if necessary to store additional values in sequence
             if ($subIdx) {
                 $idx = substr($idx, 0, -length($subIdx));   # remove old sub-index
                 $subIdx = substr($subIdx, 1) + 1;
                 $subIdx = length($subIdx) . $subIdx;
+            } elsif (@delPaths) {
+                $path = shift @delPaths;
+                # make sure new path is unique
+                while ($capture{$path}) {
+                    last unless $path =~ s/ \d(\d+)$/' '.length($1+1).($1+1)/e;
+                }
+                next;
             } else {
                 $subIdx = '10';
             }
@@ -1228,7 +1357,6 @@ sub WriteXMP($$;$)
 # write out the new XMP information (serialize it)
 #
     # start writing the XMP data
-    my $useShorthand = $et->Options('XMPShorthand');
     my (@long, @short, @resFlag);
     $long[0] = $long[1] = $short[0] = '';
     if ($$et{XMP_NO_XPACKET}) {
@@ -1291,7 +1419,17 @@ sub WriteXMP($$;$)
             my $uri = $nsUsed{$1};
             unless ($uri) {
                 $uri = $nsURI{$1};      # we must have added a namespace
-                $uri or $xmpErr = "Undefined XMP namespace: $1", next;
+                unless ($uri) {
+                    # (namespace prefix may be empty if trying to write empty XMP structure, forum12384)
+                     if (length $1) {
+                        my $err = "Undefined XMP namespace: $1";
+                        if (not $xmpErr or $err ne $xmpErr) {
+                            $xmpFile ? $et->Error($err) : $et->Warn($err);
+                            $xmpErr = $err;
+                        }
+                     }
+                     next;
+                }
             }
             $nsNew{$1} = $uri;
             # need a new description if any new namespaces
@@ -1306,7 +1444,7 @@ sub WriteXMP($$;$)
             my ($path2, $ns2);
             foreach $path2 (@pathList) {
                 my @ns2s = ($path2 =~ m{(?:^|/)([^/]+?):}g);
-                my $opening = 0;
+                my $opening = $compact{OneDesc} ? 1 : 0;
                 foreach $ns2 (@ns2s) {
                     next if $ns2 eq 'rdf';
                     $nsNew{$ns2} and ++$opening, next;
@@ -1335,18 +1473,17 @@ sub WriteXMP($$;$)
             # open the new description
             $prop = $rdfDesc;
             %nsCur = %nsNew;            # save current namespaces
-            my $sp = $noPad ? '' : ' ';
             my @ns = sort keys %nsCur;
-            $long[-2] .= "\n$sp<$prop rdf:about='${about}'";
+            $long[-2] .= "$nl$sp<$prop rdf:about='${about}'";
             # generate et:toolkit attribute if this is an exiftool RDF/XML output file
-            if (@ns and $nsCur{$ns[0]} =~ m{^http://ns.exiftool.ca/}) {
-                $long[-2] .= "\n$sp${sp}xmlns:et='http://ns.exiftool.ca/1.0/'" .
+            if (@ns and $nsCur{$ns[0]} =~ m{^http://ns.exiftool.(?:ca|org)/}) {
+                $long[-2] .= "\n$sp${sp}xmlns:et='http://ns.exiftool.org/1.0/'" .
                             " et:toolkit='Image::ExifTool $Image::ExifTool::VERSION'";
             }
             $long[-2] .= "\n$sp${sp}xmlns:$_='$nsCur{$_}'" foreach @ns;
             push @curPropList, $prop;
-            # set resFlag to 0 to indicate base description when XMPShorthand enabled
-            $resFlag[0] = 0 if $useShorthand;
+            # set resFlag to 0 to indicate base description when Shorthand enabled
+            $resFlag[0] = 0 if $compact{Shorthand};
         }
         my ($val, $attrs) = @{$capture{$path}};
         $debug and print "$path = $val\n";
@@ -1358,34 +1495,34 @@ sub WriteXMP($$;$)
             $prop =~ s/ .*//;       # remove list index if it exists
             # (we may add parseType and shorthand properties later,
             #  so leave off the trailing ">" for now)
-            $long[-1] .= ($noPad ? '' : ' ' x scalar(@curPropList)) . "<$prop";
+            $long[-1] .= ($compact{NoIndent} ? '' : ' ' x scalar(@curPropList)) . "<$prop";
             if ($prop ne $rdfDesc and ($propList[$n+1] !~ /^rdf:/ or
                 ($propList[$n+1] eq 'rdf:type' and $n+1 == $#propList)))
             {
                 # check for empty structure
                 if ($propList[$n+1] =~ /:~dummy~$/) {
-                    $long[-1] .= " rdf:parseType='Resource'/>\n";
+                    $long[-1] .= " rdf:parseType='Resource'/>$nl";
                     pop @curPropList;
                     $dummy = 1;
                     last;
                 }
-                if ($useShorthand) {
+                if ($compact{Shorthand}) {
                     $resFlag[$#curPropList] = 1;
                     push @long, '';
                     push @short, '';
                 } else {
                     # use rdf:parseType='Resource' to avoid new 'rdf:Description'
-                    $long[-1] .= " rdf:parseType='Resource'>\n";
+                    $long[-1] .= " rdf:parseType='Resource'>$nl";
                 }
             } else {
-                $long[-1] .= ">\n"; # (will be no shorthand properties)
+                $long[-1] .= ">$nl"; # (will be no shorthand properties)
             }
         }
         my $prop2 = pop @propList;  # get new property name
         # add element unless it was a dummy structure field
         unless ($dummy or ($val eq '' and $prop2 =~ /:~dummy~$/)) {
             $prop2 =~ s/ .*//;      # remove list index if it exists
-            my $pad = $noPad ? '' : ' ' x (scalar(@curPropList) + 1);
+            my $pad = $compact{NoIndent} ? '' : ' ' x (scalar(@curPropList) + 1);
             # (can't write as shortcut if it has attributes or CDATA)
             if (defined $resFlag[$#curPropList] and not %$attrs and $val !~ /<!\[CDATA\[/) {
                 $short[-1] .= "\n$pad$prop2='${val}'";
@@ -1397,7 +1534,7 @@ sub WriteXMP($$;$)
                     my $quot = ($attrVal =~ /'/) ? '"' : "'";
                     $long[-1] .= " $attr=$quot$attrVal$quot";
                 }
-                $long[-1] .= length $val ? ">$val</$prop2>\n" : "/>\n";
+                $long[-1] .= length $val ? ">$val</$prop2>$nl" : "/>$nl";
             }
         }
     }
@@ -1414,9 +1551,9 @@ sub WriteXMP($$;$)
         $$dirInfo{ExtendedXMP} = $rtn[0];
         $$dirInfo{ExtendedGUID} = $rtn[1];
         # compact if necessary to fit
-        $compact = 1 if length($long[-2]) + 101 * $numPadLines > $maxDataLen;
+        $compact{NoPadding} = 1 if length($long[-2]) + 101 * $numPadLines > $maxDataLen;
     }
-    $compact = 1 if $$dirInfo{Compact};
+    $compact{NoPadding} = 1 if $$dirInfo{Compact};
 #
 # close out the XMP, clean up, and return our data
 #
@@ -1438,7 +1575,7 @@ sub WriteXMP($$;$)
             # pad to specified DirLen
             if ($len > $dirLen) {
                 my $str = 'Not enough room to edit XMP in place';
-                $str .= '. Try XMPShorthand option' unless $$et{OPTIONS}{XMPShorthand};
+                $str .= '. Try Shorthand feature' unless $compact{Shorthand};
                 $et->Warn($str);
                 return undef;
             }
@@ -1448,7 +1585,7 @@ sub WriteXMP($$;$)
                 $len += length($pad) * $num;
             }
             $len < $dirLen and $long[-2] .= (' ' x ($dirLen - $len - 1)) . "\n";
-        } elsif (not $compact and not $xmpFile and not $$dirInfo{ReadOnly}) {
+        } elsif (not $compact{NoPadding} and not $xmpFile and not $$dirInfo{ReadOnly}) {
             $long[-2] .= $pad x $numPadLines;
         }
         $long[-2] .= ($$dirInfo{ReadOnly} ? $pktCloseR : $pktCloseW);
@@ -1457,14 +1594,7 @@ sub WriteXMP($$;$)
     unless (%capture or $xmpFile or $$dirInfo{InPlace} or $$dirInfo{NoDelete}) {
         $long[-2] = '';
     }
-    if ($xmpErr) {
-        if ($xmpFile) {
-            $et->Error($xmpErr);
-            return -1;
-        }
-        $et->Warn($xmpErr);
-        return undef;
-    }
+    return($xmpFile ? -1 : undef) if $xmpErr;
     $$et{CHANGED} += $changed;
     $debug > 1 and $long[-2] and print $long[-2],"\n";
     return $long[-2] unless $xmpFile;
@@ -1491,7 +1621,7 @@ This file contains routines to write XMP metadata.
 
 =head1 AUTHOR
 
-Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
